#!/usr/bin/perl
# Virtual server creation phase specific to RedHat GNU Linux.
# Copyright (c) 2000 Designer Servers Ltd.
# Contributed by Nick Burrett <nick@dsvr.net>
#
# This file is part of the Virtual Server Administrator (FreeVSD)
#
# FreeVSD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# FreeVSD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FreeVSD; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

# This script is called by the top-level vs-batch.pl (host/generic/scripts)

# os_make_vs <virtual server>
sub os_make_vs {
    my (@nam1, @nam2, $x, $y, $vs);

    $vs = $_[0];

    # Anonymous ftp
    mkdir ("$vsdir/home/ftp", 0755);
    mkdir ("$vsdir/home/ftp/incoming", 0755);

    # Create subdirectories in /var
    mkdir ("$vsdir/var/run", 0755);

    # Set the timezone
    chdir ("$vsdir/etc");
    symlink ("../usr/share/zoneinfo/Europe/London", "localtime");

    # Create the tty devices
    chdir ("$vsdir/dev");
    @nam1 = ("p".."z", "a".."e");
    @nam2 = (0..9, "a".."f");
    for ($x = 0; $x <= 15; $x++) {
	for ($y = 0; $y <=15; $y++) {
	    $minor = ($x * 16) + $y;
	    `mknod -m 0620 pty$nam1[$x]$nam2[$y] c 2 $minor`; 
	    `mknod -m 0620 tty$nam1[$x]$nam2[$y] c 3 $minor`; 
	}
    }

    # Make other useful devices
    `mknod -m 0666 zero c 1 5`;
    `rm -f $vsdir/dev/null`;
    `mknod -m 0666 null c 1 3`;
    `mknod -m 0666 urandom c 1 9`;
    `mknod -m 0666 tty c 5 0`;
    mkdir ("fd", 0755);
    symlink ("../var/run/log", "log"); # For syslogd
}

# os_config_vs <server> <ipaddr> <fqdn> <startuid> <numuid>
sub os_config_vs {
  my ($vs, $ip, $fqdn, $startuid, $numuid, $enduid);

  $vs = $_[0];
  $ip = $_[1];
  $fqdn = $_[2];
  $startuid = $_[3];
  $numuid = $_[4];
  $enduid = $startuid + $numuid - 1;

  if (!defined($childpid = fork ())) {
    die "cannot fork: $!";
  } elsif ($childpid == 0) {
    # child
    my ($gid_admin, $gid_mail, $gid_web, $gid_ftp);
    my ($uid_admin, $uid_web, $uid_ftp);
    chroot ($vsdir);
    chdir ("/");

    $gid_admin = $startuid;
    $gid_mail = $startuid + 1;
    $gid_web = $startuid + 2;
    $gid_ftp = $startuid + 3;
    
    $uid_admin = $gid_admin;
    $uid_web = $gid_web;
    $uid_ftp = $gid_ftp;
    
    `/usr/sbin/groupadd web -g $gid_web`;
    `/usr/sbin/groupadd ftp -g $gid_ftp`;
    
    `/usr/sbin/useradd -c webserver -d /home/web -u $uid_web -g web -G admin -M -n -s /bin/bash web`;
    `/usr/sbin/useradd -c ftpserver -d /home/ftp -u $uid_ftp -g ftp -M -n -s /bin/bash ftp`;

    @admin_files = ("/home/httpd", "/home/ftp", "/tmp",
		    "/var", "/root", "/etc/mail");
    chown $uid_admin, $gid_admin, @admin_files;
    `chown -R root.root /etc`;

    # Allow users to edit sendmail configuration
    `chown -R admin.admin /etc/mail`;

    # Allow users to edit /etc/logrotate.d
    `chown -R admin.admin /etc/logrotate.d`;

    # Create subdirectories of /var and /usr/local
    @subdir = ("/usr/local", "/usr/local/bin", "/usr/local/doc",
	       "/usr/local/etc", "/usr/local/games", "/usr/local/include",
	       "/usr/local/info", "/usr/local/lib", "/usr/local/man",
	       "/usr/local/sbin", "/usr/local/src",
	       "/var/catman", "/var/lib", "/var/lib/texmf",
	       "/var/lib/texmf/fonts", "/var/lib/texmf/texfonts",
	       "/var/local", "/var/lock", "/var/lock/subsys",
	       "/var/log", "/var/log/httpd", "/var/preserve",
	       "/var/run", "/var/spool", "/var/spool/at",
	       "/var/spool/cron", "/var/spool/mail", "/var/spool/mqueue");
    foreach (@subdir) {
	mkdir ("$_", 0755);
	chown $uid_admin, $gid_admin, $_;
    }
    chmod (01777, "/tmp");
    chmod (0755, "/etc");
    chmod (0755, "/etc/vsd");
    chmod (0775, "/var/catman");
    chmod (0755, "/var/lock");
    chmod (0755, "/var/spool/at");
    chmod (0775, "/var/spool/mail");

    # Certainly directories cannot be written to by admin
    `chown -R root.man /var/catman`;
    `chown -R root.uucp /var/lock`;
    `chown -R daemon.admin /var/spool/at`;

    # Create the dynamic library cache. Allow it to be re-created by
    # the admin user.
    `/sbin/ldconfig`;
    `chown admin.admin /etc/ld.so.cache`;
    
    # /etc files that the admin user should not have write access to.
    foreach $file ("login.defs", "vsd/priv", "sendmail.cw", "sendmail.ct",
		   "virtusertable", "crontab", "sendmail.cf",
		   "httpd/conf/httpd.conf", "pam.d/*") {
	chmod (0644, "/etc/$file");
    }

    # Fixup file permissions to stop sendmail complaining.
    chmod (0755, "/etc/mail");
    chmod (0755, "/etc");
    chmod (0666, "/etc/mail/ip_allow");
    chmod (0644, "/etc/aliases");
    # Generate /etc/aliases.db.
    `/usr/bin/newaliases 2>/dev/null`;
    
    # Certain files in /etc should be owned by the admin user
    @admin_files = ("/etc/aliases", "/etc/at.deny", "/etc/bashrc",
		    "/etc/hosts",
		    "/etc/issue.net", "/etc/logrotate.conf",
		    "/etc/lynx.cfg", "/etc/ld.so.conf",
		    "/etc/resolv.conf",
		    "/etc/proftpd.conf", "/etc/sendmail.cw",
		    "/etc/virtusertable", "/etc/virtusertable.db",
		    "/etc/rc.conf", "/etc/rc.local");
    push (@admin_files, glob ("/etc/httpd"));
    push (@admin_files, glob ("/etc/mail"));
    push (@admin_files, glob ("/etc/logrotate.d"));
    chown $uid_admin, $gid_admin, @admin_files;

    # Set the password here, rather than in the generic script
    # because we need to access the crypt library and the search
    # paths may not be setup until here.
    `/usr/bin/passwd -u admin -p $password`;
    exit;
  } else {
    waitpid ($childpid, 0);
  }

  # Set quotas for the users. Purely arbitary.
  `/usr/sbin/webclients quota_userset localhost $vs admin 10240K`;
  `/usr/sbin/webclients quota_userset localhost $vs web 1024K`;
  `/usr/sbin/webclients quota_userset localhost $vs mail 4096K`;
  `/usr/sbin/webclients quota_userset localhost $vs ftp 512K`;
}

$temp = 1;
