/* Re-generate a binary to execute only when the user has the necessary
   privilege.
   Copyright (c) 1999 Designer Servers Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include "libvsd.h"

int main (int argc, char *argv[])
{
  int fd, x;
  char *program, *priv, *dest;
  unsigned int buffer[32], total_len;
  FILE *handle;
  ssize_t count;
  char template[12], cmdline[256];

  if (argc != 4)
    {
      fprintf (stderr, "usage: defpriv <program> <privilege> <new program>\n");
      return 1;
    }

  program = argv[1];
  priv = argv[2];
  dest = argv[3];
  /* This is a rush job because I couldn't think of a better way to do
     it in the short time period allowed.  */
  fd = open (program, O_RDONLY);
  if (fd == -1)
    {
      fprintf (stderr, "defpriv: could not open %s: %s\n",
	       program, strerror (errno));
      return 1;
    }

  strcpy (template, "wrpXXXXXX");
  mktemp (template);
  strcat (template, ".c");

  handle = fopen (template, "wb");
  if (handle == NULL)
    {
      fprintf (stderr, "defpriv: couldn't open %s for writing: %s\n",
	       dest, strerror (errno));
      close (fd);
      return 1;
    }

  total_len = 0;

  fprintf (handle, "unsigned long program_data[] = {\n"); 
  while ((count = read (fd, buffer, sizeof (buffer))))
    for (x = 0; x < count / 4; x++)
      {
	total_len += 4;
	fprintf (handle, "0x%x,\n", buffer[x]);
      }
  fprintf (handle, "0}; \n");

  fprintf (handle, "char *privilege = \"%s\";\n", priv);
  fprintf (handle, "unsigned long program_length = 0x%x;\n\n",
	   total_len);


  fclose (handle);
  close (fd);

  sprintf (cmdline,
	   "gcc -O2 %s restrict.c -o %s -I../libvsd -L../libvsd -lvsd",
	   template, dest);
  system (cmdline);

  remove (template);
  return 0;
} 
