/* Password file manipulation.
   Copyright (c) 1999 Designer Servers Ltd Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* Passwd file manipulation functions where the file is not necessarily
   /etc/passwd.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <sys/types.h>

#include "vsd.h"
#include "libvsd.h"

struct passwd *vsd_getpwuid (FILE *stream, uid_t uid)
{
  struct passwd *pwd;

  if (stream == NULL)
    {
      errno = EINVAL;
      return NULL;
    }

  /* Return to the file beginning.  */
  fseek (stream, 0L, SEEK_SET);

  /* Search for the user.  */
  while ((pwd = fgetpwent (stream)))
    if (pwd->pw_uid == uid)
      return pwd;

  /* Not found.  */
  return NULL;
}

struct passwd *vsd_getpwnam (FILE *stream, const char *name)
{
  struct passwd *pwd;

  if (stream == NULL)
    {
      errno = EINVAL;
      return NULL;
    }

  fseek (stream, 0L, SEEK_SET);

  while ((pwd = fgetpwent (stream)))
    if (strcmp (pwd->pw_name, name) == 0)
      return pwd;

  return NULL;
}

/* If `virtual_server' is NULL then open the host server's
   /etc/passwd file.  */
FILE *vsd_openpw (struct connection *vc, const char *virtual_server)
{
  char filename[256];

  if (virtual_server == NULL)
    strcpy (filename, "/etc/passwd");
  else
    {
      sprintf (filename, "%s/etc/passwd",
	       vsd_map_server_root (vc->map, virtual_server));
    }

  return fopen (filename, "r");
}

void vsd_closepw (FILE *stream)
{
  fclose (stream);
}
