/* Module interface to the ProFTPD configuration files.
   Copyright (c) 1999 Designer Servers Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "vsd.h"
#include "libvsd.h"

#define PROFTPD_CONF "/etc/proftpd.conf"

static int scan_directive (const char *line, const char *directive,
			   char terminator, char *buffer, size_t buflen)
{
  int x;

  line = vsd_file_skip_spaces (line);

  if (strncmp (line, directive, strlen (directive)) != 0)
    return -1;

  /* Skip past directive.  */
  line += strlen (directive);
  line = vsd_file_skip_spaces (line);
  x = 0;
  while (*line && *line != terminator && x < buflen)
    buffer[x++] = *line++;

  buffer[x] = '\0';

  return 0;
} 

/* Has `username' been configured for the chrtftp privilege.  */
int module_proftpd_check_chrt_priv (struct connection *vc,
				    const char *username)
{
  struct vsd_file_line *proftp_conf, *list;
  char temp[256];

  if (vsd_load_file (&proftp_conf, vc->server_root, PROFTPD_CONF))
    {
      io_buffer_store (vc, "-ERR An error occurred: %s", strerror (errno));
      return -1;
    }

  for (list = proftp_conf; list; list = list->next)
    if (! scan_directive (list->line, "DefaultRoot", '\n',
			  temp, sizeof (temp)))
      {
	/* `temp' will contain something like "~ <username>"  */
	if (strstr (temp, username) != NULL)
	  {
	    vsd_free_file (proftp_conf);
	    return 0;
	  }
      }

  vsd_free_file (proftp_conf);
  return 1;
}

int module_proftpd_add_chrt_priv (struct connection *vc, const char *username)
{
  char temp[256];
  struct vsd_file_line *proftp_conf;

  if (vsd_load_file (&proftp_conf, vc->server_root, PROFTPD_CONF))
    {
      io_buffer_store (vc, "-ERR An error occurred: %s", strerror (errno));
      return 1;
    }

  sprintf (temp, "DefaultRoot ~ %s", username);
  vsd_file_insert_line (vsd_file_tail (proftp_conf), temp);
  vsd_save_file (proftp_conf, vc->server_root, PROFTPD_CONF);
  vsd_free_file (proftp_conf);

  return 0;
}
