/* API for interfacing with the Virtual Server administrator.
   Copyright (c) 1999 Designer Servers Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef VSD_LIB_H
#define VSD_LIB_H

#ifdef WITH_OPENSSL
#include "rsa.h"
#include "crypto.h"
#include "x509.h"
#include "pem.h"
#include "ssl.h"
#include "err.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

#define WEB_INTERFACE 1
#define CLI_INTERFACE 0

typedef struct _vsd_server
{
#ifdef WITH_OPENSSL
  SSL_CTX *ctx;
  SSL *ssl;
#endif
  int fd; /* file descriptor.  */
  int data; /* amount of data in the buffer.  */
  char *buffer; /* data buffer */
  int buffer_size; /* size of data buffer */
  int dp; /* index into data buffer.  */
  char virtual_server[32];
} vsd_server;

extern const char *vsd_response_ptr (void);
extern void vsd_set_interface (int value);
extern int vsd_interface (void);

extern vsd_server *vsd_open (const char *host_server,
			     const char *virtual_server,
			     const char *username, const char *password);

extern void vsd_close (vsd_server *server);

extern void vsd_set_error (const char *error);
extern void vsd_set_response (const char *response);
extern int vsd_reply_success (vsd_server *server);
extern char *vsd_process_reply (vsd_server *server);
extern char *vsd_get_line (vsd_server *server);
extern int vsd_send_line (vsd_server *server, const char *fmt, ...);
extern int vsd_send_file (vsd_server *server, const char *file);
extern int vsd_send_command (vsd_server *server, const char *data,
			     const char *fmt, ...);
extern int vsd_send_command_file (vsd_server *server, const char *fname,
				  const char *fmt, ...);

extern int vsd_user_list (vsd_server *server);
extern int vsd_user_control (vsd_server *server, int action,
			     const char *user_name, const char *password,
			     const char *home_dir, const char *shell,
			     const char *new_username);
extern int vsd_group_control (vsd_server *server, int action,
			      const char *group_name);

extern int vsd_set_password (vsd_server *server, const char *username,
			     const char *old_password,
			     const char *new_password);
extern int vsd_mod_rights (vsd_server *server, const char *user_name,
			   const char *rights);
extern char *vsd_geterror (void);

extern int vsd_list_rights (vsd_server *server, const char *user_name,
			    int interface);

/* Sendmail configuration file manipulation.  */
extern int vsd_virt_list_domain (vsd_server *server, const char *domain);
extern int vsd_virt_modify_table (vsd_server *server, const char *filename);

/* Apache configuration file manipulation.  */
extern int vsd_httpd_vh_enum (vsd_server *server);
extern int vsd_httpd_vh_delete (vsd_server *server, const char *virtual_host,
				const char *server_name);
extern int vsd_httpd_vh_get (vsd_server *server, const char *virtual_host,
			     const char *server_name);
extern int vsd_httpd_vh_mod (vsd_server *server, const char *virtual_host,
			     const char *server_name, const char *filename);
extern int vsd_sendmail_cw (vsd_server *server, int action,
			    const char *domain);
extern int vsd_gethostservername (char *buffer, int size);

#ifdef __cplusplus
}
#endif

#endif
