/* Web backend for the virtual server administrator, for administrators.
   Copyright (c) 1999 Designer Servers Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdarg.h>
#include <sys/syslog.h>

#include "vsd_lib.h"
#include "module.h"

char *program_name;

static void error (int usage,
		   const char *fmt, ...) __attribute__ ((__noreturn__));

static void error (int usage, const char *fmt, ...)
{
  va_list ap;

  va_start (ap, fmt);
  if (usage == 1)
    fprintf (stdout, "usage: %s ", program_name);

  vfprintf (stdout, fmt, ap);
  va_end (ap);
  fputc ('\n', stdout);

  exit (1);
}

int main (int argc, char *argv[])
{
  vsd_server *server;
  char host_server[256], *virtual_server;
  char *operation = argv[1];
  const char *st;

  openlog ("webadmclients", LOG_PID, LOG_DAEMON);
  vsd_set_interface (WEB_INTERFACE);

  if (argc <= 1 || operation == NULL)
    {
      module_help (ADMIN_HOST, NULL);
      return 1;
    }

  syslog (LOG_WARNING, "cmd: %s %s %s %s %s %s %s %s %s",
	  argv[0], argv[1], argv[2], argv[3], argv[4],
	  argv[5], argv[6], argv[7], argv[8], argv[9]);
  
  program_name = argv[0];

  /* Might be a module.  */
  st = module_syntax (ADMIN_HOST, operation, argc - 2, argv + 1);
  if (st != NULL)
    {
      if (strcmp (st, "notexist") == 0)
	error (0, "command %s does not exist", operation);
      else
	error (1, "%s", st);
    }

  /* Get the host server and virtual server names.  */
  strcpy (host_server, argv[2]);
  virtual_server = argv[3];
  server = vsd_open (host_server, NULL, "root", "NOAUTH");

  if (server == NULL)
    error (0, "connection failed: %s", vsd_geterror ());

  /* Perform the action on the virtual server.  */
  st = module_command (server, ADMIN_HOST, operation, argv + 2);
  if (st && strncmp (st, "-ERR", 4) == 0)
    {
      fprintf (stdout, "error: %s\n", st);
      vsd_close (server);
      return 1;
    }

  fprintf (stdout, "%s", st);
  vsd_close (server);
  return 0;
}
